// --- [ hyt_221_log.c ] ------------------------------------------------------
//

#include <avr/io.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "TWI_Master_c.h"
#include "hyt_221_log.h"
#include "global.h"
#include "rs232_input.h"

void Make_Dez_Point(void);

// ----------------------------------------------------------------------------
// liest die Werte aus dem Modul aus -
// falls die Daten noch nicht bereit sind, dann starten weitere Leseversuche.
// Die eingelesenen Wert werden umgerechnet und mit 1 Nachkommastelle binr in data abgelegt
// Gibt die Anzahl der gelesen Datenbytes zurck:
// 4 - wenn Feuchtigkeit und Temperatur gemessen wurde
// 2 - wenn nur Feuchtigkeit gemessen wurde (da sind dann noch nderungen erforderlich !)
// 0 - wenn der Sensor nicht gefunden wurde

uint8_t HYT_221_read(uint8_t * data)
{
uint32_t t;
int16_t  temperatur;
uint16_t humidity;
uint8_t  buffer[6];
uint8_t  i = 10;

while(1)
	{
	buffer[0] = HYT221_ADR;						// Adresse fr TWI-Write
	TWI_MA_Start_Transceiver_With_Data(buffer, 1);

// Testen, ob das Modul geantwortet hat
	if (!(TWI_LastTransOK()))					// Prfen, ob Slave ein ACK gesendet hat
		{
//		puts_p(PSTR("TWI-Write-Error"));
//		put_c(13);

		return(0);
		}

	delay_10ms(10);								// Wartezeit 100ms (50ms reicht nicht aus)

	buffer[0]++;									// Adresse fr TWI-Read incrementieren
	TWI_MA_Start_Transceiver_With_Data(buffer, 5); // Daten via TWI einlesen
	TWI_MA_Get_Data_From_Transceiver(buffer, 5);// Daten abholen

/*
// dieser Test kann auch entfallen
	if (!(TWI_LastTransOK()))
		{
		puts_p(PSTR("TWI-Read-Error"));
		put_c(13);
		return;
		}
*/
	if (buffer[1]  & 0b11000000) 				// Bit.7/6 des ersten Datenbytes testen
		{
		i--;											// wenn gesetzt, dann ist der Sensor noch busy
//		put_c('+');									// ein '+' plus Flags hexadezimal ausgeben
//		put_s(utoa((buffer[1] & 0b11000000), cv, 16));
//		put_c(' ');									// wenn diese Ausgabe erfolgt, dann war die Wartezeit zu kurz !
		}

	else												// wenn Bit.7/6 nicht gesetzt sind
		{
		break;										// die Schleife while(1) beenden
		}

	if (!(i))										// wenn i-Versuche erfolglos waren,
		{												// dann beenden

//		puts_p(PSTR("Modul busy"));
//		put_c(13);
		return(0);										// Abbrechen
		}
	}
/*
// -------------------------------------  zur Kontrolle die Messwerte ausgeben
for (i = 1; i < 5; i++)							// ohne die TWI-Adresse in buffer[0] !
	{
	put_s(utoa(buffer[i], cv, 10));
	put_c(' ');
	}

put_c(':');
put_c(' ');
*/

// ---- hier die gelesenen Bytes uninterpretiert speichern
i = 0;
data[i++] = (uint8_t) (buffer[1]);
data[i++] = (uint8_t) (buffer[2]);
data[i++] = (uint8_t) (buffer[3]);
data[i]   = (uint8_t) (buffer[4]);
return(4);


// ------------------------------------- 	Humidity konvertieren
t  = (buffer[1] << 8);
t +=  buffer[2];
t *= 1000UL;										// mit 100 * 10 multiplizieren fr
t /= 16384UL;										// eine zustzliche Nachkommastelle !

//t += 5;												// Nachkommastelle runden
//humidity = (uint16_t) (t / 10);
humidity = (uint16_t) (t);


utoa(humidity, cv, 10);							// die Zahl in String in cv[] umwandeln

// und ein Dezimaltrennzeichen einfgen
i = 0;
while(cv[i]) i++;									// zum Stringende durchhangeln
cv[i + 1] = 0;
cv[i] = cv[i-1];
cv[i-1] = SEPARATOR;

put_s(cv);											// Wert ausgeben
put_c(' ');


// ------------------------------------- Temperatur konvertieren
t  = buffer[3] << 8;
t += (buffer[4] & 0b00000011);
t *= 165;
t /= 16384L;
//t += 5;												// Nachkommastelle runden
//t /= 10;
temperatur = (int16_t) (t - 40);


itoa(temperatur, cv, 10);						// Die Zahl in String in cv[] umwandeln

// und ein Dezimaltrennzeichen einfgen
i = 0;
while(cv[i]) i++;
cv[i + 1] = 0;
cv[i] = cv[i-1];
cv[i-1] = SEPARATOR;

put_s(cv);											// Wert ausgeben
put_c(13);


// hier die Daten binr speichern
//put_c(' ');

/*
i = 0;
data[i++] = (uint8_t) (humidity >> 8);
data[i++] = (uint8_t) (humidity);
data[i++] = (uint8_t) (temperatur >> 8);
data[i]   = (uint8_t) (temperatur);
*/



/*
for (i = 0; i < 4; i++)							// zur Kontrolle ausgeben
	{
	put_s(utoa(buffer[i], cv, 10));
	put_c(' ');
	}

put_c(13);
DF_write_array(buffer, 4);
*/
return(4);											// Rckgabe der Anzahl geschriebener Bytes

}


// ----------------------------------------------------------------------------
void HYT_221_konvert(uint8_t * data)
{
uint8_t i,j = 0;
uint16_t h;
int16_t t;

h  = data[j++] << 8;
h += data[j++];

utoa(h, cv, 10);							// die Zahl in String in cv[] umwandeln

/*
// und ein Dezimaltrennzeichen einfgen
i = 0;
while(cv[i]) i++;									// zum Stringende durchhangeln
cv[i + 1] = 0;
cv[i] = cv[i-1];
cv[i-1] = DEZIMAL_PUNKT;
*/

put_c(SEPARATOR);
put_s(cv);											// Wert ausgeben

t  = data[j++] << 8;
t += data[j];

/*
itoa(t, cv, 10);						// Die Zahl in String in cv[] umwandeln

// und ein Dezimaltrennzeichen einfgen
i = 0;
while(cv[i]) i++;
cv[i + 1] = 0;
cv[i] = cv[i-1];
cv[i-1] = DEZIMAL_PUNKT;
*/
itoa(t, cv, 10);

put_c(SEPARATOR);
put_s(cv);											// Wert ausgeben

}

/*
// ----------------------------------------------------------------------------
// fgt zwischen dem letzten und dem vorletzten Zeichen im String cv[]
// einen Dezimalpunkt ein
// kostet bei 2-maligem Aufruf aber mehr Speicher als inline
void Make_Dez_Point(void)
{
uint8_t i = 0;

while(cv[i]) i++;									// zum Stringende durchhangeln
cv[i + 1] = 0;										// den neuen Terminator setzen
cv[i] = cv[i-1];									// das letzte Zeichen nach rechts kopieren
cv[i-1] = SEPARATOR;								// das Dezimaltrennzeichen einfgen
}
*/

// --- [ end of file ] --------------------------------------------------------